#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "ttftool.h"
#define	TRUE	1
#define	FALSE	0
void get_namestring(TT_Face face, TT_UShort platformID, int j, char *name_string) {
  char *p, buffer[5];
  int src, dst, euc, s;
  TT_UShort size;

  TT_Get_Name_String(face, j, &p, &size);
  for(src = dst = 0;src < size;src++, dst++) {
    switch(platformID) {
    case 1:
      if(p[src] & 0x80) {
        euc = sjis2euc((((int)p[src] & 0xff) << 8) + ((int)p[src + 1] & 0xff));
        name_string[dst] = (euc >> 8) & 0xff;
        name_string[++dst] = euc & 0xff;
        src++;
      } else if(p[src] >= 0x20) {
        name_string[dst] = p[src];
      } else {
        name_string[dst] = '.';
      }
      break;
    case 3:
      if(p[src] == 0) {
        name_string[dst] = p[++src];
      } else {
        euc = utf2euc((((int)p[src] & 0xff) << 8) + ((int)p[src + 1] & 0xff));
        if(euc == 0) {
          buffer[0] = p[src + 1] + 0x40;
          buffer[1] = buffer[2] = 0;
          if(p[src + 2] == '\377') buffer[1] = p[src + 3] + 0x40;
          s = han2zen(&euc, buffer);
          euc |= 0x8080;
          if(s == 2) src += 2;
        }
        src++;
        name_string[dst] = (euc >> 8) & 0xff;
        name_string[++dst] = euc & 0xff;
      }
      break;
      default:
        name_string[dst] = p[src];
    }
  }
  name_string[dst] = '\0';
}

void get_name_properties(TT_Face face, int index, name_properties *name) {
    TT_Get_Name_ID(face, index, &(name->platformID), &(name->encodingID),
                                &(name->languageID), &(name->nameID));
}

char* get_platform(name_properties name) {
    static char data[][16] = {"Apple Unicode", "Macintosh", "ISO", "Microsoft", "Unknown"};
    static char datajp[][32] = {"Apple Unicode", "ޥåȥå", "ISO", "ޥե", "Unknown"};

    if(name.platformID >= 4) return data[4];
    if(name.languageID == 11 || name.languageID == 0x0411)
      return datajp[name.platformID];
    else
      return data[name.platformID];
}

char* get_encoding(name_properties name) {
    static char apple[][32] = {"UniCode default", "UniCode 1.1", 
                               "UniCode ISO-10646.1993", "UniCode 2.0", "Unknown"};
    static char mac[][32] =  {"Roman", "JIS", "Chinese", "Korean", "Other"};
    static char iso[][32] =  {"ASCII", "ISO-10646", "ISO8859", "Unknown"};
    static char ms[][64] =  {"Symbol", "Unicode", "Shift-JIS", "GB2312", "Big 5", "WanSung", "Johab", "Unknown"};

    switch(name.platformID) {
    case 0:
      if(name.encodingID >= 4) return apple[4];
      return apple[name.encodingID];
    case 1:
      if(name.encodingID >= 4) return mac[4];
      return mac[name.encodingID];
    case 2:
      if(name.encodingID >= 3) return iso[3];
      return iso[name.encodingID];
    case 3:
      if(name.encodingID >= 7) return ms[7];
      return ms[name.encodingID];
    default:
      return apple[4];
    }
}

char* get_language(name_properties name) {
    static char buffer[32];

    strcpy(buffer, "Unknown");
    switch(name.platformID) {
    case 0:
    case 2:
      sprintf(buffer, "Unknown %d", name.languageID);
      break;
    case 1:
    case 3:
      if(name.languageID < 0x400) {
        switch(name.languageID) {
        case 0:
          strcpy(buffer, "English"); break;
        case 11:
          strcpy(buffer, "Japanese"); break;
        case 19:
          strcpy(buffer, "Chinese"); break;
        case 23:
          strcpy(buffer, "Korean"); break;
        default:
          sprintf(buffer, "Other Language %d", name.languageID);
        }
      } else {
        switch(name.languageID) {
        case 0x0404:
          strcpy(buffer, "Chinese - Taiwan"); break;
        case 0x0804:
          strcpy(buffer, "Chinese - China"); break;
        case 0x0c04:
          strcpy(buffer, "Chinese - HongKong"); break;
        case 0x409:
          strcpy(buffer, "English - United States"); break;
        case 0x809:
          strcpy(buffer, "English - United Kingdom"); break;
        case 0x0411:
          strcpy(buffer, "Japanese - Japan"); break;
        case 0x412:
          strcpy(buffer, "Korean - Korea"); break;
        default:
          sprintf(buffer, "Other Language %d", name.languageID);
        }
      }
    }
    return buffer;
}

char* get_desc(name_properties name) {
    static char desc[][64] = {
      "Copyright notice",
      "Font Family name",
      "Font Subfamily name",
      "Unique font identifier",
      "Full font name",
      "Version string",
      "Postscript name for the font",
      "Trademark",
      "Manufacturer Name",
      "Designer",
      "Description",
      "URL Vendor",
      "URL Designer",
      "License Description",
      "License Info URL",
      "Reserved",
      "Preferred Family (Windows only)",
      "Preferred Subfamily (Windows only)",
      "Compatible Full (Macintosh only)"};
    static char descjp[][64] = {
      "ɽ",
      "եȤΥեߥ꡼̾",
      "եȤեߥ꡼̾",
      "եͭμ̻",
      "եȤΥե͡",
      "Сʸ",
      "ݥȥץȤΥե̾",
      "ɸ",
      "̾",
      "ǥʡ",
      "",
      "ԤURL̾",
      "ǥʡURL̾",
      "饤󥹵",
      "饤󥹤˴ؤURL̾",
      "Reserved",
      "Preferred Family (Windows only)",
      "Preferred Subfamily (Windows only)",
      "Compatible Full (Macintosh only)"};

    if(name.nameID >= 19) return desc[19];
    if(name.languageID == 11 || name.languageID == 0x0411)
      return descjp[name.nameID];
    else
      return desc[name.nameID];
}

char* get_vendor(TT_OS2 os2) {
    static char vend[][48] = {
      "ACG ", "Agfa",
      "ADBE", "Adobe",
      "AGFA", "AGFA Compugraphic",
      "ALTS", "misc",
      "APPL", "Apple",
      "B&H ", "Bigelow Holmes",
      "BERT", "Berthold",
      "BITM", "Bitmap Software",
      "BITS", "Bitstream",
      "C&C ", "Carter Cone",
      "CANO", "Canon",
      "CTDL", "China Type Design Ltd.",
      "DSCI", "Design Science Inc.",
      "DYNA", "DynaLab",
      "DTC ", "Digital Typeface Corp.",
      "EDGE", "Rivers Edge Corp.",
      "EGT ", "ARPHIC TECHNOLOGY",
      "ELSE", "Elseware",
      "EPSN", "Epson",
      "ERAM", "Eraman",
      "FBI ", "The Font Bureau Inc",
      "FJ  ", "Fujitsu",
      "GALA", "Gal E1pagos Design Group Inc",
      "GLYF", "Glyph Systems",
      "GPI ", "Gamma Productions Inc",
      "HP  ", "Hewlett Packard",
      "HY  ", "Han Yang System",
      "IBM ", "IBM",
      "IDF ", "International Digital Fonts",
      "IMPR", "Impress",
      "JUST", "Just System",
      "KATF", "Kigsley ATF",
      "LANS", "Lanston Type",
      "LEAF", "Interleaf Inc",
      "LETR", "Letraset",
      "LINO", "Linotype",
      "LP  ", "LetterPerfect Fonts",
      "LTRX", "Lighttracks",
      "MACR", "Macromedia",
      "MLGC", "Micrologic Software",
      "MONO", "Monotype",
      "MS  ", "Microsoft",
      "MT  ", "Monotype",
      "NEC ", "NEC",
      "PARA", "ParaGraph Intl",
      "PRFS", "Production First Software",
      "QMSI", "QMS Imagen",
      "RICO", "Ricoh",
      "SFUN", "Soft Union",
      "SOHO", "Soft Horizons",
      "SWFT", "Swfte International",
      "TILD", "SIA Tilde",
      "URW ", "URW",
      "VLKF", "Visualogik Technology Design",
      "ZSFT", "Zsoft",
      "UNKN", "misc"};
    static char err[8] = "misc";
    int i;

/*putchar(os2.achVendID[0]);
putchar(os2.achVendID[1]);
putchar(os2.achVendID[2]);
putchar(os2.achVendID[3]);*/
    for(i = 0;strncmp(vend[i], "UNKN", 4) != 0;i += 2) {
      if(strncmp(vend[i], os2.achVendID, 4) == 0) return vend[i + 1];
    }
    return err;
}

void get_type1_line(FILE* fp, char* buffer) {
    int	c, i;
    
    i = 0;
    c = ' ';
    while(isspace(c)) c = getc(fp);
    while(c != '\n' && c != '\r') {
      buffer[i++] = c;
      c = getc(fp);
    }
    buffer[i] = '\0';
}

int  get_type1_string(char *fn, type1_properties *font) {
    FILE	*fp;
    float	itn;
    int		i, j, c;
    char	buffer[512], buf2[128], fullpost[126];

    font->uniqid = 0;
    fp = fopen(fn, "r");
    if (fp == NULL) return FALSE;
    for(i = 0;i < 6;i++) getc(fp);
    for(i = 0;i < 17;i++) {
      buffer[i] = c = getc(fp);
      if(c == '\n' || c == '\r') break;
    }
    if(strncmp(buffer, "%!PS-AdobeFont-1.0", 17) != 0 &&
       strncmp(buffer, "%!FontType1-1.0", 14) != 0) {
      fclose(fp);
      return FALSE;
    }
    while(c != '\n' && c != '\r') c = getc(fp);
    font->fontname[0] = font->vendor[0] = font->weight[0] = '\0';
    font->fullname[0] = fullpost[0] = '\0';
    while(strncmp(buffer, "currentdict end", 15) != 0) {
      get_type1_line(fp, buffer);
      if(strncmp(buffer, "/FullName", 9) == 0) {
      	for(i = 9;buffer[i] != '(';i++);
      	for(i++, j = 0;buffer[i] != ')';i++, j++) {
      	  font->fullname[j] = (buffer[i] == '-') ? ' ' : buffer[i];
      	  font->vendor[j] = (buffer[i] == ' ' || buffer[i] == '-') ? '\0' : buffer[i];
      	  fullpost[j] = (buffer[i] == ' ') ? '-' : buffer[i];
      	}
      	fullpost[j] = font->fullname[j] = font->vendor[j] = '\0';
      }
      if(strncmp(buffer, "/FamilyName", 11) == 0) {
      	for(i = 11;buffer[i] != '(';i++);
      	for(i++, j = 0;buffer[i] != ')';i++, j++) {
      	  font->fontname[j] = (buffer[i] == '-') ? ' ' : buffer[i];
        }
      	font->fontname[j] = '\0';
      }
      if(strncmp(buffer, "/Weight", 7) == 0) {
      	for(i = 7;buffer[i] != '(';i++);
      	for(i++, j = 0;buffer[i] != ')';i++, j++) font->weight[j] = buffer[i];
      	font->weight[j] = '\0';
      	if(strcmp(font->weight, "Plain") == 0 ||
      	   strcmp(font->weight, "Medium") == 0 ||
      	   strcmp(font->weight, "Regular") == 0) strcpy(font->weight, "normal");
      }
      if(strncmp(buffer, "/isFixedPitch", 13) == 0) {
      	sscanf(&buffer[13], "%s", buf2);
        font->pitch = (strcmp(buf2, "false") == 0) ? 'p' : 'c';
      }
      if(strncmp(buffer, "/ItalicAngle", 12) == 0) {
      	sscanf(&buffer[12], "%f", &itn);
        font->italic = (itn == 0.0) ? 'r' : 'i';
      }
      if(strncmp(buffer, "/UniqueID", 9) == 0) {
      	sscanf(&buffer[9], "%d", &(font->uniqid));
      }
      if(strncmp(buffer, "/FontName", 9) == 0) {
      	sscanf(&buffer[9], "%s", font->postname);
        j = strlen(font->postname);
        for(i = 0;i < j;i++) font->postname[i] = font->postname[i + 1];
      }
    }
    if(font->postname[0] == '\0') strcpy(font->postname, fullpost);
    if(strcmp(font->fontname, "Computer Modern") == 0 || 
       strcmp(font->fontname, "LaTeX") == 0) {
      strcpy(font->vendor, font->fontname);
      strcpy(font->fontname, font->fullname);
    }
    if(strstr(font->fullname, "Italic") != NULL) font->italic = 'i';
    fclose(fp);
    return TRUE;
}
